import {Component, OnInit} from '@angular/core';
import {ActivatedRoute, Router} from '@angular/router';
import {SpeciesAnalysis} from 'src/app/models/SpeciesAnalysis';
import {DataService} from 'src/app/services/data.service';

import {SLFDatabase} from "../../models/SLFDatabase";

@Component({
  selector: 'slf-family',
  templateUrl: './family.component.html',
  styleUrls: ['./family.component.scss']
})
export class FamilyComponent implements OnInit {
  public readonly id: string;
  private SLFDatabase?: SLFDatabase;
  public speciesNames?: string[];
  public speciesUrlSpecie?: string[];
  public currentSpeciesName: string = '';
  public currentSpeciesResults: SpeciesAnalysis[] = [];
  public readonly displayedColumns = ['species', 'result', 'actions'];

  constructor(
    private route: ActivatedRoute,
    private readonly router: Router,
    private dataService: DataService
  ) {
    const paramMap = this.route.snapshot.paramMap;
    if (!paramMap.has('id')) {
      this.router.navigate(['/']);
      throw new Error('Missing id');
    }
    this.id = <string>paramMap.get('id');
  }

  public ngOnInit(): void {
    this.dataService.getMultipleFamilyAnalysis()
      .subscribe(slfd => {
        this.SLFDatabase = slfd;
        console.log(slfd)
        this.speciesNames = this.extractSpeciesNames(slfd.families[this.id].results);
        this.speciesUrlSpecie=Object.keys(slfd.families[this.id].results)
        if (this.speciesNames !== undefined){
          this.changeCurrentSpecies(this.speciesNames[0]);
        }
      });
  }

  public extractResultId(speciesAnalysis: SpeciesAnalysis): string {
    const name = speciesAnalysis.directory;
    return name.substring(name.lastIndexOf('/') + 1);
  }

  private changeCurrentSpecies(speciesNameParam: string): void {
    let cont=0;
    let speciesAnalysisList: SpeciesAnalysis[] = [];
    if (this.SLFDatabase !== undefined) {
      this.currentSpeciesName = speciesNameParam;
      Object.keys(this.SLFDatabase.families[this.id].results).forEach((key: string) => {
        if (this.SLFDatabase !== undefined) {
          if(this.SLFDatabase.families[this.id].results[key].speciesName.includes(speciesNameParam)){
            speciesAnalysisList.push(this.SLFDatabase.families[this.id].results[key]);
            cont++;
          }
      }
      

    })

    this.currentSpeciesResults=speciesAnalysisList;
    
    } else {
      console.error('this.multipleFamilyAnalysis is undefined');
    }
  }

  private extractSpeciesNames(data){
    let species:string[]=[];
    let keys=Object.keys(data);
    keys.forEach((key) => {
      if(key.includes(this.id)){
        let urlSpecieName=data[key].speciesName;
        let index=urlSpecieName.indexOf(this.id);
        if(!species.includes(urlSpecieName.substring(0,index-1))){
          species.push(urlSpecieName.substring(0,index-1));
        }
      }
  }); 
  return species;
  }

  public onChangeSpecies(speciesName: string): void {
    let flag=false;
    let speciesUrl=speciesName;
    if(this.speciesUrlSpecie!==undefined){
      for (var i = 0; i < this.speciesUrlSpecie.length; i++) {
       if((this.speciesUrlSpecie[i].includes(speciesName))&&(!flag)){
        speciesUrl=this.speciesUrlSpecie[i];
       }
    }
    }
    this.changeCurrentSpecies(speciesName);
  }

  public formatResultName(resultName){
    let index=resultName.indexOf(this.id);
    index+=this.id.length+1;
    return resultName.substring(index,resultName.length);
  }

}
