import {Injectable} from '@angular/core';
import {HttpClient} from '@angular/common/http';
import {environment} from "../../environments/environment";
import {SLFDatabase} from "../models/SLFDatabase";
import {BehaviorSubject, filter, map, Observable} from "rxjs";

@Injectable({
  providedIn: 'root'
})
export class DataService {
  private readonly SLFDatabase: BehaviorSubject<SLFDatabase | undefined>;
  private readonly newick: BehaviorSubject<string | undefined>;

  constructor(private readonly http: HttpClient) {
    this.SLFDatabase = new BehaviorSubject<SLFDatabase | undefined>(undefined);
    this.newick = new BehaviorSubject<string | undefined>(undefined);
  }

  public getMultipleFamilyAnalysis(): Observable<SLFDatabase> {
    return <Observable<SLFDatabase>>this.SLFDatabase
      .asObservable()
      .pipe(filter(value => value !== undefined));
  }

  public listFamilyNames(): Observable<string[]> {
    if (this.SLFDatabase === undefined) {
      throw new Error("multipleFamilyAnalysis is not initialized");
    }

    return this.getMultipleFamilyAnalysis()
      .pipe(map(slfd => Object.keys(slfd.families)));
  }

  public getNewick(): Observable<string> {
    return <Observable<string>>this.newick.asObservable()
      .pipe(filter(value => value !== undefined));
  }

  public loadNewick(): void {
    if (this.newick.value !== undefined) {
      throw new Error("newick is already initialized");
    }

    this.http.get<{ newick: string }>(`${environment.apiUrl}/families`)
      .pipe(map(result => result.newick))
      .subscribe(newick => this.newick.next(newick));
  }

  public loadAnalysisResults(): void {
    if (this.SLFDatabase.value !== undefined) {
      throw new Error("multipleFamilyAnalysis is already initialized");
    }

    this.http.get<SLFDatabase>(`${environment.apiUrl}/database`)
      .subscribe(slfd => this.SLFDatabase.next(slfd));
  }
}
